/*  mexSVMTrain.cpp

    MATLAB Mex C++ code for SVM training.

    This provides an MATLAB interface of LIBSVM 2.33.

    Refer to http://www.csie.ntu.edu.tw/~cjlin for more information
      on LIBSVM.

    By Yi Zhao, OSU-EE, 01/20/2002 (version 2.00).
    By Yi Zhao, OSU-EE, 12/31/2001 (version 1.10).
    By Yi Zhao, OSU-EE, 12/28/2001 (version 1.00).

    Modifiedy by Junshui Ma, 02/12/2002 to fit in the 
	the OSU SVM framework.
*/


// MEX
#include "mex.h"
// LIBSVM
#include "svm.h"
// mzSVM common module
#include "mzSVMCommon.h"

// Verbose level.
int mzVerbose;

//added by junshui, 
double svmInfo_epsilon_nu = 0;
double svmInfo_nu = 0;
double svmInfo_C = 0;
double svmInfo_object = 0;
int svmInfo_iter_num = 0;


// mexFunction : Gateway function to MATLAB.
//
//Matlab Function:
// [AlphaY, SVs, Bias, Parameters] = mexSVMTrain(Samples, Labels)
// [AlphaY, SVs, Bias, Parameters, nSV, nLabel] = mexSVMTrain(Samples, Labels, Parameters)
// [AlphaY, SVs, Bias, Parameters, nSV, nLabel] = mexSVMTrain(Samples, Labels, Parameters, Weight)
// [AlphaY, SVs, Bias, Parameters, nSV, nLabel] = mexSVMTrain(Samples, Labels, Parameters, Weight, Verbose)
//
// Inputs:
//    Samples    - training patterns, MxN;
//    Labels     - labels of training patterns, 1xN;
//    Parameters - input parameters;
//    Weight     - weighting factors of patterns 1xA, where A<=L;
//    Verbose    - verbose level
//                0 : very silent (default);
//                1 : a little verbose;
//                2 : very verbose.
// Outputs:
//    AlphaY    - coef. of SVs, (L-1) x sum(nSV);
//    SVs       - SVs, M x sum(nSV);
//    Bias      - bias of classifier(s), 1 x L*(L-1)/2;
//    Parameters -  parameters used in training.
//    nSV       -  numbers of SV in each class, 1xL;
//    nLabel    -  Labels of each class, 1xL;

void mexFunction(int nlhs, mxArray * plhs[], int nrhs,
		const mxArray * prhs[])
{
	svm_problem prob;
	svm_parameter parame;
	svm_model * model;
	svm_node * x_space;
	double *X, *Y, *inPar, *WT, *dVerbose;
	double *dLabels, *dnSVs, *SVs, *alphaY, *Bias, *outPar;
	int mX, nX, nY, nPar, nWT;
	int nClass, nSV, nBias;
	
	// Test the input arguments.
	if (nrhs < 2 || nrhs > 5)
		mexErrMsgTxt("mexSVMTrain: Wrong number of input arguments!\n");
	if(nrhs < 5)
		mzVerbose = 0;
	else {
		dVerbose  = mxGetPr(prhs[4]);
		mzVerbose = (int) dVerbose[0];
	}


	if(mzVerbose>0)
		printf("mexSVMTrain:\n");

	// Read problem.
	if(mzVerbose>0)
		printf("1. Read problem ... ");
	X  = (double *) mxGetPr(prhs[0]);	// patterns
	mX = mxGetM(prhs[0]);
	nX = mxGetN(prhs[0]);
	Y  = (double *) mxGetPr(prhs[1]);	// labels
	nY = mxGetN(prhs[1]);
	if (nX!=nY)
		mexErrMsgTxt("mexSVMTrain: Sizes of samples and labels do not match. \n");
	x_space = mzSVMReadProblem(&prob, mX, nX, X, Y);
	if(mzVerbose>0)
		printf("OK!\n");

	// Read input paremeters.
	if(mzVerbose>0)
		printf("2. Read parameters ... ");
	mzSVMInitParameter(&parame);
	if(nrhs >= 3) {
		if(mxGetM(prhs[2])!=1)
			mexErrMsgTxt("mexSVMTrain: parameters is not a row vector!");
		inPar = mxGetPr(prhs[2]);
		nPar  = mxGetN(prhs[2]);
		mzSVMReadParameter(&parame, inPar, nPar);
	}
	if(parame.gamma == 0)
		parame.gamma = 1.0/mX;
	if(mzVerbose>0)
		printf("OK!\n");
	
	// Read weighting coefficients if available.
	if(nrhs >= 4) {
		if(mxGetM(prhs[3])!=0 && mxGetN(prhs[3])!=0) {
			WT = mxGetPr(prhs[3]);
			nWT = mxGetM(prhs[3]);
			if(nWT==1)
				nWT = mxGetN(prhs[3]);
			else
				if(mxGetN(prhs[3])==1)
					mexErrMsgTxt("mexSVMTrain: WT is not vector!");
			mzSVMReadWeight(&parame, WT, nWT);
		}
	}

	// Train SVM classifer(s).
	if(mzVerbose>0)
		printf("3. Train SVM model ... ");
	model = svm_train(&prob, &parame);
	if(mzVerbose>0)
		printf("OK!\n");

	// Create output arguments.
	nClass = model->nr_class;
	nSV    = model->l;
	nBias  = nClass * (nClass-1) / 2;


	// Create output arguments.
	nClass = model->nr_class;
	nSV    = model->l;
	nBias  = nClass * (nClass-1) / 2;
	plhs[0] = mxCreateDoubleMatrix(nClass-1, nSV, mxREAL);	// SV coef. or AlphaY.
	alphaY  = mxGetPr(plhs[0]);
	plhs[1] = mxCreateDoubleMatrix(mX, nSV, mxREAL);	// SVs
	SVs     = mxGetPr(plhs[1]);
	plhs[2] = mxCreateDoubleMatrix(1, nBias, mxREAL);	// Bias
	Bias    = mxGetPr(plhs[2]);
	plhs[3] = mxCreateDoubleMatrix(1, 11, mxREAL);	// Parameters
	outPar  = mxGetPr(plhs[3]);
	plhs[4] = mxCreateDoubleMatrix(1, nClass, mxREAL);	// nSV
	dnSVs   = mxGetPr(plhs[4]);
	plhs[5] = mxCreateDoubleMatrix(1, nClass, mxREAL);	// nLabel
	dLabels = mxGetPr(plhs[5]);

	// Save output arguments.
	if(mzVerbose>0)
		printf("4. Output SVM model ... ");
	// Save model to output arguments.
	mzSVMSaveModel(model, dLabels, dnSVs, SVs, alphaY, Bias, mX);
	// Save output parameters.
	mzSVMSaveParameter(&(model->param), outPar);

	//added by Junshui
	if (svmInfo_C != 0.0)
		outPar[4] = svmInfo_C;
	if (svmInfo_nu != 0.0)
		outPar[8] = svmInfo_nu;
	if (svmInfo_epsilon_nu != 0.0)
		outPar[9] = svmInfo_epsilon_nu;




	if(mzVerbose>0)
		printf("OK!\n");

	// Free memory used in model.
	if(mzVerbose>0)
		printf("5. Clear memory ... ");
	svm_destroy_model(model);
	if(mzVerbose>0)
		printf("OK!\n");

	// Free temporary memory.
	mxFree(prob.y);
	mxFree(prob.x);
	mxFree(x_space);
}
